// include the library code:
#include <SoftwareSerial.h>
#include <LiquidCrystal.h>
#include <dht11.h>

//lcd(12, 11, 5, 4, 3, 2);

#define motorFan    6

#define butMode     8
#define butSx       9
#define butDx       7

#define livelSensor A0  // sensore temperatura al pin analogico A0
#define DHT11PIN    10
#define ledPin 13

void ChangeMenu();
void ChangeClock();
void DisplayMenu();
void MenuDefault();
void MenuMedia();
void ValoriAggiornati();
void MenuValori();
void MenuSetClock();
void MenuSendValues();
void PrintClock(int col);
void SaveValue();
void SaveDay();
void SendValues();
void Clock();
void SalvaValori();

//Variabili globali
int menu = 0;
boolean mainMenu;

int sensorValue = 0;
int reading = 0;
int sensorPin = A0;
byte sensorValues[144];
byte sensorValues_t[144];

int sensorLastValue;
int sensorLastValue_t;

int levelSensorValue = 0;
boolean ledblink = false;

boolean buttonStateSx = 0;         // current state of the button
boolean lastButtonStateSx = 0;     // previous state of the button
boolean buttonStateDx = 0;         // current state of the button
boolean lastButtonStateDx = 0;     // previous state of the button
boolean buttonStateMode = 0;         // current state of the button
boolean lastButtonStateMode = 0;     // previous state of the button

byte countDay;
byte day1[144];
byte day2[144];
byte day1_t[144];
byte day2_t[144];

byte settingClock;
byte settingSerial;
unsigned long previousTime = 0;
byte seconds;
byte minutes;
byte hours;
byte days;


int ledState = LOW;             // ledState used to set the LED
unsigned long previousMillis = 0;        // will store last time LED was updated
const long interval = 1000;           // interval at which to blink (milliseconds)


// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

dht11 DHT11;
SoftwareSerial NSS(0, 5);

void setup() {

  pinMode(butSx, INPUT);
  pinMode(butDx, INPUT);
  pinMode(butMode, INPUT);
  pinMode(livelSensor, INPUT);

  digitalWrite (motorFan, OUTPUT);
  digitalWrite (ledPin, OUTPUT);

  mainMenu = true;
  settingClock = 0;

  seconds = 0;
  minutes = 0;
  hours = 0;
  days = 0;

  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);

  lcd.setCursor(0, 0);
  lcd.print("loading ...");

  Serial.begin(9600);

  NSS.begin(9600);

  delay(1000);  // Pausa di stabilizzazione sensore

  sensorLastValue = 0;
  SaveValue();

  delay(1000);  // Pausa di stabilizzazione sensore
  lcd.clear();
}

void loop() {

  // Menu normale
  if (mainMenu == true) {
    ChangeMenu();
  } else {
    switch (menu) {
      case 1:
        ValoriAggiornati();
        break;
      case 2:
        SalvaValori();
        break;
      case 3:
        ChangeClock();
        break;
      case 4:
        SendValues();
        break;
    }
  }

  DisplayMenu(); // Mostra il display

  // Aggiorna l'orologio
  if (menu != 3) {
    Clock();
  }

  // Ventolina
  if (sensorLastValue > 65) {
    digitalWrite(motorFan, 1);
  } else {
    digitalWrite(motorFan, 0);
  }

  //Led
  if (levelSensorValue > 250) {
    if (levelSensorValue > 400) {
      digitalWrite(ledPin, HIGH);
    } else {
      if (ledblink == true) {
        digitalWrite(ledPin, HIGH);
      } else {
        digitalWrite(ledPin, LOW);
      }
    }
  } else {
    digitalWrite(ledPin, LOW);
  }

}

//Funzioni
void ChangeMenu() {
  boolean bntPremuto = false;
  boolean bntMode = false;
  buttonStateSx = digitalRead(butSx);
  buttonStateDx = digitalRead(butDx);
  buttonStateMode = digitalRead(butMode);

  if (buttonStateSx != lastButtonStateSx) {
    if (digitalRead(butSx) == 1) {
      bntPremuto = true;
      menu--;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  if (buttonStateDx != lastButtonStateDx) {
    if (digitalRead(butDx) == 1) {
      bntPremuto = true;
      menu++;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  if (buttonStateMode != lastButtonStateMode) {
    if (digitalRead(butMode) == 1) {
      bntPremuto = true;
      bntMode = true;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  lastButtonStateSx = buttonStateSx;
  lastButtonStateDx = buttonStateDx;
  lastButtonStateMode = buttonStateMode;

  // Se sono nel menu 3,4 (clock settings) flaggo il bit che mi fa cambiare menu'
  if (bntPremuto == true) {
    lcd.clear();
    if (menu < 0) {
      menu = 4;
    } else if (menu > 4) {
      menu = 0;
    }
    if ((bntMode == true) && (menu == 1)) {
      mainMenu = false;
    }
    if ((bntMode == true) && (menu == 2)) {
      mainMenu = false;
    }
    if ((bntMode == true) && (menu == 3)) {
      settingClock = 1;
      mainMenu = false;
    }
    if ((bntMode == true) && (menu == 4)) {
      settingSerial = 1;
      mainMenu = false;
    }
  }
}

void ChangeClock() {
  int _valore = 0;
  boolean bntPremuto = false;
  boolean bntMode = false;
  buttonStateSx = digitalRead(butSx);
  buttonStateDx = digitalRead(butDx);
  buttonStateMode = digitalRead(butMode);

  if (buttonStateSx != lastButtonStateSx) {
    if (digitalRead(butSx) == 1) {
      bntPremuto = true;
      _valore = -1;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  if (buttonStateDx != lastButtonStateDx) {
    if (digitalRead(butDx) == 1) {
      bntPremuto = true;
      _valore = +1;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  if (buttonStateMode != lastButtonStateMode) {
    if (digitalRead(butMode) == 1) {
      bntPremuto = true;
      //_valore = 0;
    }
    // Delay a little bit to avoid bouncing
    delay(50);
  }

  lastButtonStateSx = buttonStateSx;
  lastButtonStateDx = buttonStateDx;
  lastButtonStateMode = buttonStateMode;

  //In base al valore di settingClock
  // aumento/diminuisco le ore o i minuti...
  if (bntPremuto == true) {
    lcd.clear();
    if (_valore == 0) {
      settingClock++;
      if (settingClock == 3) {
        settingClock = 0;
        mainMenu = true;
        menu = 0;
      }
    } else {
      if (settingClock == 1)
      {
        hours = hours + _valore;
        if ((hours == 0) && (_valore == -1)) {
          hours = 23;
        }
        if (hours >= 24) {
          hours = 0;
        }
      } else {
        minutes = minutes + _valore;
        if ((minutes == 0) && (_valore == -1)) {
          minutes = 59;
        }
        if (minutes >= 60) {
          minutes = 0;
        }
      }
    }
  }

}

void DisplayMenu() {
  switch (menu) {
    case 0:
      MenuDefault();
      break;
    case 1:
      MenuMedia();
      break;
    case 2:
      MenuValori();
      break;
    case 3:
      MenuSetClock();
      break;
    case 4:
      MenuSendValues();
      break;
  }
}

void MenuDefault() {

  PrintClock(0);

  lcd.setCursor(0, 1);
  lcd.print("U:");
  lcd.setCursor(2, 1);
  lcd.print(sensorLastValue);

  lcd.setCursor(5, 1);
  lcd.print("T:");
  lcd.setCursor(7, 1);
  lcd.print(sensorLastValue_t);

  lcd.setCursor(10, 1);
  lcd.print("L:");
  lcd.setCursor(12, 1);
  lcd.print(levelSensorValue);

}

void MenuMedia() {
  lcd.setCursor(0, 0);
  lcd.print("Agg. sensore:");
  lcd.setCursor(0, 1);
  lcd.print("> premi mode");
}

void ValoriAggiornati() {
  SaveValue();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Dati aggiornati");
  menu = 0;
  mainMenu = true;
  delay(1000);
  lcd.clear();
}

void MenuValori() {
  lcd.setCursor(0, 0);
  lcd.print("Save day data:");
  lcd.setCursor(0, 1);
  lcd.print("> premi mode");
}

void SalvaValori() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Loading ...");
  SaveDay();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Valori salvati");
  menu = 0;
  mainMenu = true;
  delay(1000);
  lcd.clear();
}

void MenuSetClock() {
  if (settingClock == 0) {
    lcd.setCursor(0, 0);
    lcd.print("Imp. orologio:");
    lcd.setCursor(0, 1);
    lcd.print("> premi mode");
  } else {
    switch (settingClock) {
      case 1:
        lcd.setCursor(0, 0);
        lcd.print("Cambia l'ora");
        break;
      case 2:
        lcd.setCursor(0, 0);
        lcd.print("Cambia minuti");
        break;
    }
    PrintClock(1);
  }

}

void MenuSendValues() {
  if (settingSerial == 0) {   
    lcd.setCursor(0, 0);
    lcd.print("Invia valori:");
    lcd.setCursor(0, 1);
    lcd.print("> premi mode");
  } else {
  }
}

void PrintClock(int col) {

  lcd.setCursor(0, col);
  lcd.print("Ore ");

  if (hours < 10) {
    lcd.setCursor(4, col);
    lcd.print("0");
    lcd.setCursor(5, col);
    lcd.print(hours);
  } else {
    lcd.setCursor(4, col);
    lcd.print(hours);
  }

  lcd.setCursor(6, col);
  lcd.print(":");

  if (minutes < 10) {
    lcd.setCursor(7, col);
    lcd.print("0");
    lcd.setCursor(8, col);
    lcd.print(minutes);
  } else {
    lcd.setCursor(7, col);
    lcd.print(minutes);
  }

  lcd.setCursor(9, col);
  lcd.print(":");

  if (seconds < 10) {
    lcd.setCursor(10, col);
    lcd.print("0");
    lcd.setCursor(11, col);
    lcd.print(seconds);
  } else {
    lcd.setCursor(10, col);
    lcd.print(seconds);
  }

}

void Clock() {
  if (millis() >= (previousTime))
  {
    previousTime = previousTime + 1000;
    seconds = seconds + 1;

    if (seconds == 60)
    {
      seconds = 0;
      minutes = minutes + 1;
      checkSensor();
      if (minutes % 10 == 0) {
        SaveValue();
      }
    }
    if (minutes == 60)
    {
      minutes = 0;
      hours = hours + 1;
    }
    if (hours == 23)
    {
      hours = 0;
      SaveDay();
      countDay++;
      if (countDay == 2) {
        countDay = 0;
      }
    }

    if (ledblink == true) {
      ledblink = false;
    } else {
      ledblink = true;
    }

  }
}

void checkSensor() {
  uint8_t chk = DHT11.read(DHT11PIN);
  sensorLastValue = DHT11.humidity;
  sensorLastValue_t = DHT11.temperature;
  levelSensorValue = analogRead(livelSensor);
}

void SaveValue() {
  byte i = 0;
  checkSensor();
  i = (hours * 6) + (minutes / 10);
  sensorValues[i] = sensorLastValue;
  sensorValues_t[i] = sensorLastValue_t;
}

void SaveDay() {
  switch (countDay) {
    case 0:
      for (byte i = 0; i < 144; i++) {
        day1[i] = sensorValues[i];
        day1_t[i] = sensorValues_t[i];
      }
      break;
    case 1:
      for (byte i = 0; i < 144; i++) {
        day2[i] = sensorValues[i];
        day2_t[i] = sensorValues_t[i];
      }
      break;
  }
}

void SendValues() {

  if (Serial.available() > 0) {
    SaveDay();
    lcd.setCursor(0, 0);
    lcd.print("Trasmissione ...");
    lcd.setCursor(0, 1);
    lcd.print("in corso");
    byte x = 0;
    for (x = 0; x < 2; x++) {
      switch (x) {
        case 0:
          for (byte i = 0; i < 144; i++) {
            Serial.println(i);
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println("day1-umi");
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println(day1[i]);
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println("day1-t");
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println(day1_t[i]);
            delay(50); // Delay a little bit to avoid bouncing
          }
          break;
        case 1:
          for (byte i = 0; i < 144; i++) {
            Serial.println(i);
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println("day2-umi");
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println(day2[i]);
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println("day2-t");
            delay(50); // Delay a little bit to avoid bouncing
            Serial.println(day2_t[i]);
            delay(50); // Delay a little bit to avoid bouncing
          }
          break;
      }
    }
    lcd.clear();
    lcd.setCursor(0, 1);
    lcd.print("valori inviati");
    settingSerial = 0;
  } else {
    lcd.setCursor(0, 0);
    lcd.print("Trasmissione ...");
    lcd.setCursor(0, 1);
    lcd.print("no connessione");
  }

  delay(1000);
  lcd.clear();
  settingSerial = 0;
  mainMenu = true;
}

